

import invocationParametersISF.IndirectSelectionFacilityInvocationParameterModel;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;

import sourceSymbolSet.SourceSymbol;

/**
 * This class implements a widget which provides services to manage the display
 * of a text string. The text string will be changed in real-time. It will be
 * composed character-by-character.
 * 
 * The real estate allocated to this component may be subjected to external
 * constraints (in height and/or width).
 * 
 * The choice about the font that is to be used may not be within the purview of
 * this component and the choice may be subjected to external constraints (min
 * size).
 * 
 * Given the combination of real estate and font constraints, it may be the case
 * that the entire text gloss may not fit in the displayable area.
 * 
 * This class distinguishes between the entire gloss and the displayed portion
 * of the gloss
 * 
 * @author mb
 * 
 */
public class CustomizedTextDisplayer extends JPanel {

	private static Font DEFAULT_FONT = new Font("serif", Font.PLAIN, 16);
	// private static Font DEFAULT_FONT = new Font("serif", Font.PLAIN, 300);

	// private Font displayFont;
	private List<SourceSymbol> sourceSymbolSetToBeDisplayed;

	// private JTextArea theTextArea;
	private JTextDisplayerWithFontInformation theTextArea;

	private IndirectSelectionFacilityInvocationParameterModel paramModel;

	private String mostRecentlyAppended;

	private boolean IS_VERBOSE = true;

	private CustomizedTextDisplayer() {
	}

	public CustomizedTextDisplayer(
			IndirectSelectionFacilityInvocationParameterModel paramManager) {
		this.setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
		this.setAlignmentX(Component.LEFT_ALIGNMENT);
		// theTextArea = new JTextArea();
		this.paramModel = paramManager;
		theTextArea = new JTextDisplayerWithFontInformation(paramManager);
		// theTextArea.setBackground(Color.WHITE);
		// theTextArea.setBackground(Color.RED);
		resetText();

		JScrollPane scrollPane = new JScrollPane(theTextArea);
		scrollPane
				.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
		scrollPane
				.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPane.setBackground(Color.YELLOW);
		// scrollPane.setBorder(null);

		// scrollPane.setBorder(new LineBorder(Color.GREEN, 2));

		this.add(scrollPane);
		// this.add(new TextDisplayerWithFontInformation());
		theTextArea.grabFocus();
		// if (isFontToBeSetAsLargeAsPossible) {
		// getTheTextArea().setDisplayFontAsLargesAsPossible(
		// this.sourceSymbolSetToBeDisplayed);
		// }

		// augment("hi");

	}

	public void paint(Graphics g) {
		// System.out.println("Painting: " + this.getClass().getName());
		// if (isFontToBeSetAsLargeAsPossible) {
		// getTheTextArea().setDisplayFontAsLargesAsPossible(
		// this.sourceSymbolSetToBeDisplayed);
		// }
		super.paint(g);
	}

	public void augmentGloss(String str) {
		mostRecentlyAppended = str;
		getTheTextArea().insertStringImmediatelyAfterCaret(str);
		getTheTextArea().repaint();
	}

	// public void setFont(Font f) {
	// getTheTextArea().setFont(f);
	// }

	public void resetText() {
		getTheTextArea().resetTheText();
	}

	public String getTheText() {
		// return this.getText().substring(0, this.getText().length() - 1)
		// .toLowerCase();
		return getTheTextArea().getTheText();
	}

	public void removeCharImmediatelyBeforeCaret() {
		getTheTextArea().removeCharacterImmediatelyBeforeCaret();
	}

	/**
	 * 
	 * @param sourceSymbolSet
	 *            the set of symbols that may potentially be displayed on this
	 *            component
	 */
	public void setDisplayFontAsLargesAsPossible(
			List<SourceSymbol> sourceSymbolSet) {

		if (IS_VERBOSE)
			System.out.println(this.getClass().getName()
					+ " : in setDisplayFontAsLargesAsPossible(-)...");

		this.sourceSymbolSetToBeDisplayed = sourceSymbolSet;

		theTextArea
				.setDisplayFontAsLargesAsPossible(sourceSymbolSetToBeDisplayed);

		// getTheTextArea().setDisplayFontAsLargesAsPossible(sourceSymbolSet);
		// Font optFont = this.getDisplayFont().deriveFont(
		// (float) this.getMaxPointSize((int) Math.ceil(this
		// .getSpecifiedSize().getHeight()), sourceSymbolSet));
		// System.out.println("Optimal Font for Display is: " + optFont);
		// System.out.println("Baseline: " + optFont.getBaselineFor('Q'));
		// getTheTextArea().setFont(optFont);
	}

	public void setPreferredSize(Dimension dim) {
		super.setPreferredSize(dim);
		theTextArea.setPreferredSize(dim);
	}

	public void setMaximumSize(Dimension dim) {
		super.setMaximumSize(dim);
		theTextArea.setMaximumSize(dim);
	}

	public void setMinimumSize(Dimension dim) {
		super.setMinimumSize(dim);
		theTextArea.setMinimumSize(dim);
	}

	public Font getDisplayFont() {
		return getTheTextArea().getFont();
	}

	// public void setDisplayFont(Font displayFont) {
	// this.displayFont = displayFont;
	// getTheTextArea().setFont(this.displayFont);
	// }

	public JTextDisplayerWithFontInformation getTheTextArea() {
		return theTextArea;
	}

	public void repositionCaret(int newPosition) {
		theTextArea.repositionCaret(newPosition);
	}

	public void clearGloss() {
		theTextArea.clearGloss();
	}

	public void showFontHints() {
		theTextArea.showFontHints();
	}

	public String getMostRecentlyAppended() {
		return mostRecentlyAppended;
	}

	public void tailorSize(Dimension size, List<SourceSymbol> sourceSymbolSet) {
		Dimension totalRealEstate = size;
		Dimension keyboardRealEstate = new Dimension(totalRealEstate.width,
				(int) Math.floor(totalRealEstate.height
						* paramModel.getKeyboardRealEstateProportion()));
		Dimension compositionWidgetRealEstate = new Dimension(
				totalRealEstate.width, totalRealEstate.height
						- keyboardRealEstate.height);

		// **** preferred size for composition widget does the trick, but not
		// for onscreenkeyboard - one of min/max does it
		this.setSize(compositionWidgetRealEstate);
		this.setPreferredSize(compositionWidgetRealEstate);
		this.setMinimumSize(compositionWidgetRealEstate);
		this.setMaximumSize(compositionWidgetRealEstate);

		theTextArea.setSize(compositionWidgetRealEstate);
		theTextArea.setPreferredSize(compositionWidgetRealEstate);
		theTextArea.setMinimumSize(compositionWidgetRealEstate);
		theTextArea.setMaximumSize(compositionWidgetRealEstate);
		if (IS_VERBOSE) {
			System.out.println(this.getClass().getName()
					+ " : CompositionWidgetRealEstate: "
					+ compositionWidgetRealEstate);
		}

		setDisplayFontAsLargesAsPossible(sourceSymbolSet);

	}

}
